<?php
namespace DoThatTask\Includes;

class Common_Functions
{	
	public $__defaultMethod 		= "GET";
	public $__paypalDetail 			= [];
	public $_passwordLen 			= 8;
	public $__iconUrl 				= "";
	const BASEURL 				 	= DTT_API_URL;
	protected $_deactivateOption 	= [DTT_DO_INSTALLATION,DTT_DO_ACTIVATION_REDIRECT,DTT_ACTIVATION_ERROR_TYPE,DTT_ACTIVATION_ERROR_DATA,DTT_TOKEN,
									   DTT_PLUGIN_ACTIVATE,DTT_PLUGIN_PRIMARY_EMAIL,DTT_PLUGIN_INIT,DTT_PLUGIN_UPGRADE,DTT_PLUGIN_MINIMUM_VERSION,
									   DTT_PLUGIN_EMAIL_VERIFY_CODE];
	protected $_uninstallOption		= [DTT_SETTING,DTT_PAYPAL];
	public $_auditToken = '';
	public $_addTaskBtnForAudit 	= false;
	public $_settings				= [];

	/**
	 * Constructor
	 *
	 * @access	public
	 * @return	void
	 */
	public function __construct(){
		$this->__iconUrl = DTT_API_URL. 'images/icon.png';
		
	}

	public function getSettings(){
		$this->__defaultMethod = "GET";
		$this->_settings = json_decode($this->postUrlUsingCurl($this->webServiceUrl("settings")),true);
	}

	/**
     * JsonEncode.
     *
	 * @param array/$data,boolean/true/false
	 * @param array/$data,boolean/true/false
     * @var string
     */
	public function jsonEncode($data , $return = false)
	{
		$json = json_encode($data,JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_UNESCAPED_UNICODE);
		if($return){
			return $json;
		}
		echo $json;
		exit();
	}

	/**
	 * Set message in session on error , success and warning all type message.
	 *
	 * @access	public
	 * @param	string	$type			Raw API request string.
	 * @param	string	$msg			Raw API request string.
	 * @return	void
	 */
	public function setMsgs($type,$msg)
	{
		if($type == "success")
		{
			$_SESSION['dtt_success'] = $msg;
		}
		else
		{
			$_SESSION['dtt_error'] = $msg;
		}
	}	

	/**
	 * Get message from session and unset session.
	 *
	 * @access	public
	 * @return	string	$html 			Returns the raw string response
	 */
	public function getMsgs()
	{
		$html = '';
		
		if(isset($_SESSION['dtt_error'])){
			$error = $_SESSION['dtt_error'];
			$html = '<div id="setting-error-settings_updated" class="updated settings-error notice is-dismissible error">'.
					'<p><strong>'.$error.'.</strong></p><button type="button" class="notice-dismiss"><span class="screen-reader-text">'.
					'Dismiss this notice.</span></button></div>';
			unset($_SESSION['dtt_error']);
		}
		elseif(isset($_SESSION['dtt_success'])){
			$success = $_SESSION['dtt_success'];
			$html = '<div id="setting-error-settings_updated" class="updated settings-error notice is-dismissible">'.
					'<p><strong>'.$success.'.</strong></p><button type="button" class="notice-dismiss"><span class="screen-reader-text">'.
					'Dismiss this notice.</span></button></div>';
			unset($_SESSION['dtt_success']);
		} 
		echo $html;
	}	

	/**
	 * Get web-service url.
	 *
	 * @access	public
	 * @param	string 	$name 		Raw API name string
	 * @return	string	$url 		Returns the raw string response
	 */
	public function webServiceUrl($name)
	{
		return self::BASEURL."api/".$name;
	}

	/**
	 * CURL function to use third party API and get response.
	 *
	 * @access	public
	 * @param	string 	$url 		Raw API url string
	 * @param	array 	$aPost 		Raw API postdata array
	 * @return	string	$response 	Returns the raw string response
	 */
	public function postUrlUsingCurl($aUrl,$aPostData = array(),$aHeader = array())
	{
		if(empty($aHeader) || !isset($aHeader['AccessToken']) || (isset($aHeader['AccessToken']) && empty($aHeader['AccessToken']))){
			$aHeader['AccessToken'] = 'Bearer '.DTT_APP_TOKEN;
		}
		$args = array('method' => $this->__defaultMethod, 'sslverify' => false,'headers'=>$aHeader);
		
		if($aPostData)
		{
			$args['body'] = $aPostData;
		}
		$timeout = 90;
		//if ( ! ini_get( 'safe_mode' ) ){}
        set_time_limit( $timeout + 30 );
		$args['timeout'] = $timeout;

		$aResponse = wp_remote_request($aUrl,$args);
		$cData = wp_remote_retrieve_body( $aResponse );
		return $cData;
	}


	/**
	 * send mail to given emails .
	 *
	 * @access	public
	 * @param	array 	$aTo 		Raw API aTo array
	 * @param	string 	$subject 	Raw API subject string
	 * @param	string 	$content 	Raw API cotent string
	 * @param	optional
	 * @return	string	$content 	Returns the raw string content
	 */
	public function sendEmail($aTo,$aSubject,$aContent,$aOpts = array())
	{
		$headers = array('Content-Type: text/html; charset=UTF-8');		 
		wp_mail( $aTo, $aSubject, $aContent, $headers );
	}
	/**
	 * get admin details.
	 *
	 * @access	public
	 * @param   string  $field  (optional) param
	 * @return	array	$aVals 	Returns the raw array admin data
	 */
	public function wpDefaultValues($field='')
	{
		if ( !function_exists( 'wp_get_current_user' ) ) { 
		    require_once ABSPATH . WPINC . '/pluggable.php'; 
		} 
		$current_user = wp_get_current_user();
		$aVals = array();
		$aVals['admin_email'] = get_option('admin_email');
		if(get_option(DTT_PLUGIN_ACTIVATE,false)){
			$aVals['admin_email'] = get_option(DTT_PLUGIN_PRIMARY_EMAIL);
		}
		$aVals['user_email'] = $current_user->user_email;
		$aVals['user_role'] = isset($current_user->roles[0])?$current_user->roles[0]:'';
		$aVals['blog_url'] = get_home_url();
		if($current_user->ID>0){
			
			$aVals['admin_token'] = get_user_meta(get_current_user_id(),'admin_token',true);
			$aVals['admin_login'] = $current_user->user_login;
			$aVals['admin_fname'] = $current_user->user_firstname ? $current_user->user_firstname : $current_user->user_login;
			$aVals['admin_lname'] = $current_user->user_lastname  ? $current_user->user_lastname : $current_user->user_login;
		}
		if(!empty($field) && isset($aVals[$field])){
			return $aVals[$field];
		}
		return $aVals;
	}
	/**
	 * set email content with html tags.
	 *
	 * @access	public
	 * @param	
	 * @param	
	 * @return	string	$content 	Returns the raw string content
	 */
	public function setEmailContent($aType,$aVals = array())
	{
		$eMailContent = "<div>";
		$eMailContent .= "<p>Hello,</p>";
		if($aType == "pluginInstall")
		{			
			$eMailContent .= "<p>Thank you for installing our plugin.</p>";
		}

		$eMailContent .= "</div>";
		return $eMailContent;
	}	
    /**
	 * read new notification and mark new notifcation isread.
	 *
	 * @access	public
	 * @param	string 	$email		Raw API email atring
	 * @param	number 	$task_id	Raw API task_id numeric
	 * @return	void
	 */
    public function readNotification($email,$task_id){
    	$curlUrl = $this->webServiceUrl("task/readnotification/".$email.'/'.$task_id);
		$aResponse = $this->postUrlUsingCurl($curlUrl);
		$aJson = json_decode($aResponse);
    }
    /**
	 * Get Unopened task count from this current user.
	 *
	 * @access	public
	 * @param	string 	$email		Raw API email atring
	 * @param	number 	$task_id	Raw API task_id numeric
	 * @return	number	$count 		Returns the raw number count
	 */
    public function getUnOpnedTaskCount($email,$task_id=''){
    	if(!empty($task_id)){
    		$this->readNotification($email,$task_id);
    	}
		$curlUrl = $this->webServiceUrl("task/unopned/".$email);
		$aResponse = $this->postUrlUsingCurl($curlUrl);
		$aJson = json_decode($aResponse);
		$count = 0;
		if($aJson && $aJson->code > 0)
		{			
			$count = $aJson->count;
		}
		return $count;
	}

	/**
	 * save transaction data at web-smarter end through this function.
	 *
	 * @access	public
	 * @param	array 	$saveData	Raw API saveData array
	 * @param	array 	$extraData	Raw API extraData array
	 * @return	array	$response 	Returns the raw array response
	 */
	public function saveTransactionData($saveData,$type='subscription'){
		$this->__defaultMethod = "POST";
		$aData = $this->wpDefaultValues();
		$curlEUrl = $this->webServiceUrl("save_subscription_transaction/".$aData['admin_email'].'/'.$type);
		$aResponse = $this->postUrlUsingCurl($curlEUrl,$saveData);
		return $aResponse = json_decode($aResponse,true);
	}
	/**
	 * get PAYPAL API details from Laravel.
	 *
	 * @access	public
	 * @param	string 	$type		Raw API type string
	 * @return	array	$response 	Returns the raw array response
	 */
	public function getPaypalAPI($type='all'){
		$curlEUrl = $this->webServiceUrl("paypal_details").'?type='.$type; 
		$this->__defaultMethod = 'GET';
		$API = $this->postUrlUsingCurl($curlEUrl);
		return json_decode($API,true);
	}
	/**
	 * Get all statuses of tasks.
	 *
	 * @access	public
	 * @param	none
	 * @return	array	$response 	Returns the raw array response
	 */
	public function getTaskStatus()
	{
		$aRows = array();
		$curlUrl = $this->webServiceUrl("task_status");
		$aResponse = $this->postUrlUsingCurl($curlUrl);
		$aJson = json_decode($aResponse);
		if($aJson->status)
		{			
			$aRows = $aJson->status;
		}
		return $aRows;
	}
	/**
	 * Delete Option DoThatTask plugin.
	 * 
	 * Used to delete options
	 *
	 * @access  public
	 * @param   array $aOption
	 * @return  none
	 */
	public function deleteOption($aOption){
		if(is_array($aOption)){
			foreach ($aOption as $value) {
				delete_option($value);	
			}
		}else{
			delete_option($aOption);	
		}
	} 
	/**
	 * Add Option DoThatTask plugin.
	 * 
	 * Used to Add multiple/Single options
	 *
	 * @access  public
	 * @param   array $aOption
	 * @return  none
	 */
	public function addOption($aOption,$value=''){
		if(is_array($aOption)){
			foreach ($aOption as $key=>$value) {
				add_option($key,$value);	
			}
		}else{
			add_option($aOption,$value);	
		}
	}
	/**
	 * Deactive DoThatTask plugin.
	 * 
	 * Used to remove unnecessary things from app and WP end
	 *
	 * @access  public
	 * @param   none
	 * @return  none
	 */
	public function pluginDeactive(){
		$wpVals = $this->wpDefaultValues();
		$curlUrl = $this->webServiceUrl("users/plugin_uninstall/".$wpVals['admin_email']);
		$aResponse = $this->postUrlUsingCurl($curlUrl,[],['AccessToken'=>$wpVals['admin_token']]);
		//delete user meta
		$this->deleteUserMeta();
		//delete all option 
		$this->deleteOption($this->_deactivateOption);
	}
	/**
	 * Uninstall DoThatTask plugin.
	 * 
	 * Used to remove unnecessary things from app and WP end
	 *
	 * @access  public
	 * @param   none
	 * @return  none
	 */
	public function pluginUninstall(){
		//delete all option 
		$this->deleteOption($this->_uninstallOption);
		$this->deleteUsers(array('role'=>DTT_PLUGIN_USER_ROLE_SLUG));
		//Delete DTT plugin user role 
		$this->removeRole();
	}
	/**
	 * Destroy all DTT role users on plugin uninstall.
	 * 
	 * @access  public
	 * @param   none
	 * @return  none
	 */
	public function deleteUsers($condition=[]){
		require_once(ABSPATH.'wp-admin/includes/user.php');
		if(!empty($condition)){
			$users = get_users( $condition );
			
			foreach ( $users as $user ) {
			    $id = $user->ID;
			    wp_delete_user( $id );
			}
		}

	}
	/**
	 * Destroy user meta data .
	 * 
	 * @access  public
	 * @param   none
	 * @return  none
	 */
	public function deleteUserMeta(){
		$meta = ['admin_token'];
		//Delete all meta data
		$uId = get_current_user_id();
		foreach ($meta as $key => $value) {
			delete_user_meta(get_current_user_id(),$value);	
		}
	}
	/**
     * Add role For plugin.
     * 
     *
     * @access  public
     * @param   none
     * @return  none
     */
	public function addRole(){
		
		global $wp_roles;
		$role = get_role( DTT_PLUGIN_USER_ROLE_SLUG );
	    if(empty($role)){
		    if ( ! isset( $wp_roles ) )
		        $wp_roles = new WP_Roles();

		    $adm = $wp_roles->get_role('administrator');
		    //Adding a 'new_role' with all admin caps
		    $wp_roles->add_role(DTT_PLUGIN_USER_ROLE_SLUG ,__( DTT_PLUGIN_USER_ROLE_NAME , DTT_TEXT_DOMAIN ), $adm->capabilities);
			//add_role( DTT_PLUGIN_USER_ROLE_SLUG ,__( DTT_PLUGIN_USER_ROLE_NAME , DTT_TEXT_DOMAIN ),  $this->__userCapability);
		}
	} 
	/**
     * Remove role For plugin.
     * 
     *
     * @access  public
     * @param   none
     * @return  none
     */
	public function removeRole(){
		remove_role( DTT_PLUGIN_USER_ROLE_SLUG );
	} 
	/**
	 * Add User for task.
	 * 
	 * Used to add user for the particular task during move under in_progress status
	 *
	 * @access  public
	 * @param   array $option
	 * @return  JSON
	 */
	public function addTaskUser($option = []){
		$response = ['code'=>0];
		try{
		    $user_id = username_exists( $option['username'] );
		    if($user_id){
		    	throw new \Exception("There is a problem in access.", 1);
		    }
		    if ( !$user_id && email_exists($option['email']) == false ) {
		        $user_id = wp_create_user( $option['username'], $option['password'], $option['email'] );
		        if( !is_wp_error($user_id) ) {
		            $user = get_user_by( 'id', $user_id );
		            $user->set_role( DTT_PLUGIN_USER_ROLE_SLUG );
		            $response = ['code'=>1,'message'=>"Access has been granted."];
		        }else{
		        	throw new \Exception("There is a problem in access.", 1);
		        }
		    }else{
	        	throw new \Exception("There is a problem in access.", 1);
	        }
		}catch(\Exception $e){
			$response = ['code'=>0,'message'=>$e->getMessage()];
		}
		return $response;
	}
	/**
	 * Destroy Task user
	 *
	 * @access  public
	 * @param   string
	 * @return  JSON
	 */
	public function getAdminUser(){
		//Grab wp DB
	    global $wpdb;
	    //Get all users in the DB
	    $wp_user_search = $wpdb->get_results("SELECT ID, display_name FROM $wpdb->users ORDER BY ID");

	    //Blank array
	    $adminArray = 0;
	    //Loop through all users
	    foreach ( $wp_user_search as $userid ) {
	        //Current user ID we are looping through
	        $curID = $userid->ID;
	        //Grab the user info of current ID
	        $curuser = get_userdata($curID);
	        //Current user level
	        $user_level = $curuser->user_level;
	        //Only look for admins
	        if($user_level >= 8){//levels 8, 9 and 10 are admin
	            //Push user ID into array
	            $adminArray = $curID;
	            break;
	        }
	    }
	    return $adminArray;
	}
	public function removeTaskUser($username,$keepData = 1){
       $response = ['code'=>1];
		try{
		    $user_id = username_exists( $username );
		    if(!$user_id){
		    	throw new \Exception("There is a problem in revoke process.", 1);
		    }
		    if($keepData == 1){
			    $admin_id = $this->getAdminUser();
			 	if(!$admin_id){
			    	throw new \Exception("There is a problem in revoke process.", 1);
			    }
			    if(wp_delete_user($user_id,$admin_id)){
			    	$response = ['code'=>1,'message'=>"Access has been revoked."];
			    }else{
			    	$response = ['code'=>0];
			    	throw new \Exception("There is a problem in revoke process.", 1);	
			    }
		    }else{
		    	if(wp_delete_user($user_id)){
			    	$response = ['code'=>1,'message'=>"Access has been revoked."];
			    }else{
			    	$response = ['code'=>0];
			    	throw new \Exception("There is a problem in revoke process.", 1);	
			    }
		    }
		}catch(\Exception $e){
			$response['message'] = $e->getMessage();
		}
		return $response;
	}
	/**
	 * generate randow password for task user.
	 * 
	 *
	 * @access  public
	 * @param   none
	 * @return  string
	 */
	public function randomPassword() {
	    $alphabet = implode('', range('a','z')).implode('', range('A','Z')).implode('', range(0,9));
	    $pass = array(); //remember to declare $pass as an array
	    $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
	    for ($i = 0; $i < $this->_passwordLen; $i++) {
	        $n = rand(0, $alphaLength);
	        $pass[] = $alphabet[$n];
	    }
	    return implode($pass); //turn the array into a string
	}
	/**
	 * Get Site LOGO.
	 * 
	 * Get site logo
	 *
	 * @access  public
	 * @param   none
	 * @return  string
	 */
	public function getSiteLogo(){
		$custom_logo_id = get_theme_mod( 'custom_logo' );
		$image = wp_get_attachment_image_src( $custom_logo_id , 'full' );
		return $image[0];
	}

	/** 
	 *
	 * @access  public
	 * @param   none
	 * @return  string
	 */
	public function forceUpgradePlugin($version){
		$savedVersion = get_option(DTT_PLUGIN_MINIMUM_VERSION,'');
		if(!empty($version) && (empty($savedVersion) || (!empty($savedVersion) && $version>$savedVersion))){
			$this->addOption(DTT_PLUGIN_MINIMUM_VERSION,$version);
		}
		$redirect = true;
		if(!empty($version) && $version > DTT_VERSION){
			if (FALSE === get_option(DTT_PLUGIN_UPGRADE) && FALSE === update_option(DTT_PLUGIN_UPGRADE,FALSE)){
				$this->addOption(DTT_PLUGIN_UPGRADE,true);
			}
		}else{
			if(get_option(DTT_PLUGIN_UPGRADE, false)){
				$this->deleteOption([DTT_PLUGIN_UPGRADE]);
				$redirect = false;
			}
		}
		return $redirect;
	}
}